@@ -21,6 +21,7 @@ use Scalar::Util;
 use strict;
 use warnings;
 use MKDoc::XML::Decode;
+use Petal::I18N;
 
 
 BEGIN
@@ -100,7 +101,7 @@ our $CURRENT_INCLUDES = 0;
 
 
 # this is for CPAN
-our $VERSION = '2.07';
+our $VERSION = '2.10';
 
 
 # The CodeGenerator class backend to use.
@@ -124,6 +125,9 @@ our $MT_NS       = 'metal';
 our $MT_NS_URI   = 'http://xml.zope.org/namespaces/metal';
 our $MT_NAME_CUR = 'main';
 
+# translation service, optional
+our $TranslationService = undef;
+
 
 # Displays the canonical template for template.xml.
 # You can set $INPUT using by setting the PETAL_INPUT environment variable.
@@ -333,18 +337,19 @@ sub process
     
     # ok, from there on we need to override any global variable with stuff
     # that might have been specified when constructing the object
-    local $TAINT              = defined $self->{taint}              ? $self->{taint}              : $TAINT;
-    local $ERROR_ON_UNDEF_VAR = defined $self->{error_on_undef_var} ? $self->{error_on_undef_var} : $ERROR_ON_UNDEF_VAR;
-    local $DISK_CACHE         = defined $self->{disk_cache}         ? $self->{disk_cache}         : $DISK_CACHE;
-    local $MEMORY_CACHE       = defined $self->{memory_cache}       ? $self->{memory_cache}       : $MEMORY_CACHE;
-    local $MAX_INCLUDES       = defined $self->{max_includes}       ? $self->{max_includes}       : $MAX_INCLUDES;
-    local $INPUT              = defined $self->{input}              ? $self->{input}              : $INPUT;
-    local $OUTPUT             = defined $self->{output}             ? $self->{output}             : $OUTPUT;
+    local $TAINT              = defined $self->{taint}               ? $self->{taint}               : $TAINT;
+    local $ERROR_ON_UNDEF_VAR = defined $self->{error_on_undef_var}  ? $self->{error_on_undef_var}  : $ERROR_ON_UNDEF_VAR;
+    local $DISK_CACHE         = defined $self->{disk_cache}          ? $self->{disk_cache}          : $DISK_CACHE;
+    local $MEMORY_CACHE       = defined $self->{memory_cache}        ? $self->{memory_cache}        : $MEMORY_CACHE;
+    local $MAX_INCLUDES       = defined $self->{max_includes}        ? $self->{max_includes}        : $MAX_INCLUDES;
+    local $INPUT              = defined $self->{input}               ? $self->{input}               : $INPUT;
+    local $OUTPUT             = defined $self->{output}              ? $self->{output}              : $OUTPUT;
     local $BASE_DIR           = defined $self->{base_dir} ? do { ref $self->{base_dir} ? undef : $self->{base_dir} } : $BASE_DIR;
     local @BASE_DIR           = defined $self->{base_dir} ? do { ref $self->{base_dir} ? @{$self->{base_dir}} : () } : @BASE_DIR;
-    local $LANGUAGE           = defined $self->{default_language}   ? $self->{default_language}   : $LANGUAGE;
-    local $DEBUG_DUMP         = defined $self->{debug_dump}         ? $self->{debug_dump}         : $DEBUG_DUMP;
-    local $DECODE_CHARSET     = defined $self->{decode_charset}     ? $self->{decode_charset}     : $DECODE_CHARSET;
+    local $LANGUAGE           = defined $self->{default_language}    ? $self->{default_language}    : $LANGUAGE;
+    local $DEBUG_DUMP         = defined $self->{debug_dump}          ? $self->{debug_dump}          : $DEBUG_DUMP;
+    local $DECODE_CHARSET     = defined $self->{decode_charset}      ? $self->{decode_charset}      : $DECODE_CHARSET;
+    local $TranslationService = defined $self->{translation_service} ? $self->{translation_service} : $TranslationService;
     # local $ENCODE_CHARSET     = defined $self->{encode_charset}     ? $self->{encode_charset}     : $ENCODE_CHARSET;
     
     # prevent infinite includes from happening...
@@ -364,8 +369,10 @@ sub process
 	die "\$hash is undefined\n\n" unless $hash;
 	$res = $coderef->($hash);
     };
-    
-    $res = $self->_handle_error ($@) if (defined $@ and $@);
+   
+    if (defined $@ and $@) { $res = $self->_handle_error ($@) }
+    elsif (defined $TranslationService) { $res = Petal::I18N->process ($res) } 
+
     return $res;
 }
 
@@ -691,19 +698,6 @@ sub _canonicalize
 Petal - Perl Template Attribute Language - TAL for Perl!
 
 
-=head1 IMPORTANT NOTE 
-
-From version 2.00 onwards Petal *requires* that you use well-formed XML. This
-is because Petal now uses L<MKDoc::XML::TreeBuilder> rather than
-L<HTML::TreeBuilder> and L<XML::Parser>.
-
-In particular, this version of Petal *CAN* break backwards compatibility if you
-were using Petal's HTML mode will non well formed XHTML.
-
-If you still want to use broken XHTML, you can Petal 2.00 in conjunction with
-L<Petal::Parser::HTB> which has been created for this purpose.
-
-
 =head1 SYNOPSIS
 
 in your Perl code:
@@ -776,46 +770,9 @@ some_object() is a subroutine that returns some kind of object, may it be a scal
 object, array referebce or hash reference. Let's see what we can do...
 
 
-=head2 Version 1: Prototype
+=head2 Version 1: WYSIWYG friendly prototype.
 
-    <!--? This is a template comment.
-          It will not appear in the output -->
-    <html xmlns:tal="http://purl.org/petal/1.0/">
-      <body>
-        This is the variable 'my_var' : ${my_var}.
-      </body>
-    </html>
-
-
-And if C<my_var> contained I<Hello World>, Petal would have outputted:
-
-    <html>
-      <body>
-        This is the variable 'my_var' : Hello World.
-      </body>
-    </html>
-
-
-Now let's say that C<my_var> is a hash reference as follows:
-
-    $VAR1 = { hello_world => 'Hello, World' }
-
-
-To output the same result, you would write:
-
-    This is the variable 'my_var' : ${my_var/hello_world}.
-
-
-=head2 Version 2: WYSIWYG friendly.
-
-The problem with the above page is that when you edit it with a WYSIWYG editor,
-or simply open it in your browser, you will see:
-
-    This is the variable 'my_var' : ${my_var/hello_world}.
-
-Ideally you don't want your graphic designers to worry about
-variable names... and that's where TAL kicks in. Using TAL
-you can do:
+Using TAL you can do:
 
     This is the variable 'my_var' :
     <span tal:replace="my_var/hello_world">Hola, Mundo!</span>
@@ -825,26 +782,23 @@ frontpage, dreamweaver, adobe golive...) and work with less risk of damaging
 your petal commands.
 
 
-=head2 Version 3: Object-oriented version
+=head2 Version 2: Object-oriented version
 
 Let's now say that C<my_var> is actually an object with a method hello_world()
-that returns I<Hello World>. To output the same result, your line:
+that returns I<Hello World>. To output the same result, your line, which was:
 
     <span tal:replace="my_var/hello_world">Hola, Mundo!</span>
 
-Would become:
-
-    <span tal:replace="my_var/hello_world">Hola, Mundo!</span>
-
-Look carefully at those two lines. That's right. There are identical. Petal
-lets you access hashes and objects in an entirely transparent way.
+Would need to be... EXACTLY the same. Petal lets you access hashes and objects
+in an entirely transparent way and tries to automagically do The Right Thing
+for you.
 
 This high level of polymorphism means that in most cases you can maintain your
 code, swap hashes for objects, and not change a single line of your template
 code.
 
 
-=head2 Version 4: Personalizable
+=head2 Version 3: Personalizable
 
 Now let's say that your method some_object() can take an optional
 argument so that C<$my_var->hello_world ('Jack')> returns I<Hello Jack>.
@@ -876,7 +830,7 @@ You cannot write nested expressions such as:
 This will NOT work. At least, not yet.
 
 
-=head2 Version 5: Internationalized
+=head2 Version 4: Internationalized
 
 Let's say that you have a directory called C<hello_world> with the following
 files:
@@ -897,6 +851,10 @@ What will happen is that the C<$template> object will try to find a file named
 C<fr-CA>, then C<fr>, then will default to <en>. It should work fine for
 includes, too!
 
+NOTE: There is now *EXPERIMENTAL, LIMITED* support for ZPT-like i18n
+attributes, which should provide a much nicer framework. See L<Petal::I18N> for
+details.
+
 
 TIP:
 
@@ -1038,26 +996,6 @@ sub-templates) are in the character set I<charset>.
 I<charset> can be any character set that can be used with the module L<Encode>. 
 
 
-=head2 Global Variables
-
-If you want to use an option throughout your entire program and don't want to
-have to pass it to the constructor each time, you can set them globally. They
-will then act as defaults unless you override them in the constructor.
-
-  $Petal::BASE_DIR           (use base_dir option)
-  $Petal::INPUT              (use input option)
-  $Petal::OUTPUT             (use output option)
-  $Petal::TAINT              (use taint option)
-  $Petal::ERROR_ON_UNDEF_VAR (use error_on_undef_var option)
-  $Petal::DISK_CACHE         (use disk_cache option)
-  $Petal::MEMORY_CACHE       (use memory_cache option)
-  $Petal::MAX_INCLUDES       (use max_includes option)
-  $Petal::LANGUAGE           (use default_language option)
-  $Petal::DEBUG_DUMP         (use debug_dump option)
-    # $Petal::ENCODE_CHARSET     (use encode_charset option) -- _DEPRECATED_
-  $Petal::DECODE_CHARSET     (use decode_charset option)
-
-
 =head1 TAL SYNTAX
 
 This functionality is directly and shamelessly stolen from the excellent TAL
@@ -1259,20 +1197,6 @@ statements will be executed in the following order:
         content
 
 
-=head2 aliases
-
-On top of all that, for people who are lazy at typing the following
-aliases are provided (although I would recommend sticking to the
-defaults):
-
-  * tal:define     - tal:def, tal:set
-  * tal:condition  - tal:if
-  * tal:repeat     - tal:for, tal:loop, tal:foreach
-  * tal:attributes - tal:att, tal:attr, tal:atts
-  * tal:content    - tal:inner
-  * tal:replace    - tal:outer
-
-
 TRAP:
 
 Don't forget that the default prefix is C<petal:> NOT C<tal:>, until
@@ -1281,68 +1205,10 @@ you set the petal namespace in your HTML or XML document as follows:
     <html xmlns:tal="http://purl.org/petal/1.0/">
 
 
-=head1 INCLUDES
+=head1 METAL MACROS
 
-Let's say that your base directory is C</templates>,
-and you're editing C</templates/hello/index.html>.
-
-From there you want to include C</templates/includes/header.html>
-
-
-=head2 general syntax
-
-You can use a subset of the XInclude syntax as follows:
-
-  <body xmlns:xi="http://www.w3.org/2001/XInclude">
-    <xi:include href="/includes/header.html" />
-  </body>
-
-
-For backwards compatibility reasons, you can omit the first slash, i.e.
-
-  <xi:include href="includes/header.html" />
-
-
-=head2 relative paths
-
-If you'd rather use a path which is relative to the template itself rather
-than the base directory, you can do it but the path MUST start with a dot,
-i.e.
-
-  <xi:include href="../includes/header.html" />
-
-  <xi:include href="./subdirectory/foo.xml" />
-
-etc.
-
-
-=head2 limitations
-
-The C<href> parameter does not support URIs, no other tag than C<xi:include> is
-supported, and no other directive than the C<href> parameter is supported at
-the moment.
-
-Also note that contrarily to the XInclude specification Petal DOES allow
-recursive includes up to C<$Petal::MAX_INCLUDES>. This behavior is very useful
-when templating structures which fit well recursive processing such as trees,
-nested lists, etc.
-
-You can ONLY use the following Petal directives with Xinclude tags:
-
-  * on-error
-  * define
-  * condition
-  * repeat
-
-C<replace>, C<content>, C<omit-tag> and C<attributes> are NOT supported in
-conjunction with XIncludes.
-
-
-=head1 MORE INCLUDES: METAL
-
-Petal now supports a subset of the METAL specification, which is a very WYSIWYG
-compatible way of doing includes. The current implementation supports only two
-metal statements: define-macro and use-macro.
+Petal supports an implementation of the METAL specification, which is a very
+WYSIWYG compatible way of doing template includes.
 
 
 =head2 define-macro
@@ -1381,6 +1247,54 @@ For example:
   </html>
 
 
+=head2 define-slot
+
+In any given macro you can define slots, which are bits of macros that can be
+overriden by something else using the fill-macro directive. To re-use the
+example above, imagine that we want to be able to optionally override the
+(pouet pouet) bit with something else:
+
+
+  File foo.xml
+  ============
+
+  <html>
+    <body>
+      <p metal:define-macro="footer">
+        (c) Me (r)(tm) <span metal:define-slot="pouet">(pouet pouet)</span>
+      </p>
+    </body>
+  </html>
+
+
+=head2 fill-slot
+
+Your including file can override any slot using the fill-slot instruction, i.e.
+
+  File bar.xml
+  ============
+
+  <html>
+    <body>
+      ... plenty of content ...
+
+      <p metal:use-macro="foo.xml#footer">
+        Page Footer. <span metal:fill-slot="pouet" petal:omit-tag="">(bar baz)</span>
+      </p>
+    </body>
+  </html>
+
+This would result in the macro 'foo.xml#footer' to produce:
+
+  <html>
+    <body>
+      <p>
+        (c) Me (r)(tm) (bar baz)
+      </p>
+    </body>
+  </html>
+
+
 =head2 self includes
 
 In Zope, METAL macros are expanded first, and then the TAL instructions are processed.
@@ -1586,142 +1500,6 @@ expressions which invoke methods with parameters, i.e.
   string:The current CGI 'action' param is: ${cgi/param --action}
 
 
-=head1 UGLY SYNTAX
-
-For certain things which are not doable using TAL you can use what
-I call the UGLY SYNTAX. The UGLY SYNTAX is UGLY, but it can be handy
-in some cases.
-
-For example consider that you have a list of strings:
-
-    $my_var = [ 'Foo', 'Bar', 'Baz' ];
-    $template->process (my_var => $my_var, buz => $buz);
-
-
-And you want to display:
-
-  <title>Hello : Foo : Bar : Baz</title>
-
-Which is not doable with TAL without making the XHTML invalid.
-With the UGLY SYNTAX you can do:
-
-    <title>Hello<?for name="string my_var"?> : <?var name="string"?><?end?></title>
-
-Of course you can freely mix the UGLY SYNTAX with other Petal
-syntaxes. So:
-
-    <title><?for name="string my_var"?> $string <?end?></title>
-
-Mind you, if you've managed to read the doc this far I must confess
-that writing:
-
-    <h1>$string</h1>
-
-instead of:
-
-    <h1 tal:replace="string">Dummy</h1>
-
-is UGLY too. I would recommend to stick with TAL wherever you can.
-But let's not disgress too much.
-
-
-=head2 variables
-
-Abstract
-
-  <?var name="EXPRESSION"?>
-
-Example
-
-  <title><?var name="document/title"?></title>
-
-Why?
-
-Because if you don't have things which are replaced by real values in your
-template, it's probably a static page, not a template... :) 
-
-
-=head2 if / else constructs
-
-Usual stuff:
-
-  <?if name="user/is_birthay"?>
-    Happy Birthday, $user/real_name!
-  <?else?>
-    What?! It's not your birthday?
-    A very merry unbirthday to you! 
-  <?end?>
-
-You can use C<condition> instead of C<if>, and indeed you can use modifiers:
-
-  <?condition name="false:user/is_birthay"?>
-    What?! It's not your birthday?
-    A very merry unbirthday to you! 
-  <?else?>
-    Happy Birthday, $user/real_name!
-  <?end?>
-
-Not much else to say!
-
-
-=head2 loops
-
-Use either C<for>, C<foreach>, C<loop> or C<repeat>. They're all the same
-thing, which one you use is a matter of taste. Again no surprise:
-
-  <h1>Listing of user logins</h1>
-  <ul>
-    <?repeat name="user system/list_users"?>
-      <li><?var name="user/login"?> :
-          <?var name="user/real_name"?></li>
-    <?end?>
-  </ul>
-  
-
-Variables are scoped inside loops so you don't risk to erase an existing
-C<user> variable which would be outside the loop. The template engine also
-provides the following variables for you inside the loop:
-
-  <?repeat name="foo bar"?>
-    <?var name="repeat/index"?>  - iteration number, starting at 0
-    <?var name="repeat/number"?> - iteration number, starting at 1
-    <?var name="repeat/start"?>  - is it the first iteration?
-    <?var name="repeat/end"?>    - is it the last iteration?
-    <?var name="repeat/inner"?>  - is it not the first and not the last iteration?
-    <?var name="repeat/even"?>   - is the count even?
-    <?var name="repeat/odd"?>    - is the count odd?
-  <?end?>
-
-Again these variables are scoped, you can safely nest loops, ifs etc...  as
-much as you like and everything should be fine.
-
-
-=head2 includes
-
-The XInclude syntax should be preferred over this...
-
-  <?include file="include.xml"?>
-
-It will include the file 'include.xml', using the current C<@Petal::BASE_DIR>
-directory list.
-
-If you want use XML::Parser to include files, you should make sure that
-the included files are valid XML themselves... FYI XML::Parser chokes on
-this:
-
-    <p>foo</p>
-    <p>bar</p>
-
-But this works:
-
-    <div>
-      <p>foo</p>
-      <p>bar</p>
-    </div>
-
-(Having only one top element is part of the XML spec).
-
-
 =head1 ADVANCED PETAL
 
 
@@ -1864,6 +1642,7 @@ The cycle of a Petal template is the following:
     6. Petal turns the perl code into a subroutine
     7. Petal::Cache::Memory caches the subroutine in memory
     8. Petal executes the subroutine
+    9. (optional) Petal internationalizes the resulting output.
 
 If you are under a persistent environement a la mod_perl, subsequent calls to
 the same template will be reduced to step 8 until the source template changes.
@@ -1917,17 +1696,6 @@ mailing list or at L<mailto://jhiver@mkdoc.com>.
 None.
 
 
-=head1 KNOWN BUGS
-
-The XML::Parser wrapper only cannot expand entities C<&lt;>, C<&gt;>, C<&amp;>
-and C<&quot;>. Besides, I can't get it to NOT expand entities in 'Stream' mode.
-
-HTML::TreeBuilder expands all entities, hence &nbsp;s are lost / converted to
-whitespaces.
-
-XML::Parser is deprecated and should be replaced by SAX handlers at some point.
-
-
 =head1 AUTHOR
 
 Copyright 2003 - MKDoc Holdings Ltd.
